# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models
from django.contrib.auth.models import User
from django.utils import timezone as utils_timezone

class Profile(models.Model):
    """
    User profile information
    """

    # Link back to the user
    user = models.OneToOneField(
        User,
        on_delete=models.CASCADE,
        related_name='profile',
        help_text='User for whom the profile is associated')

    # Address for the user. This is so that they can change it if they want
    address = models.CharField(
        max_length=256,
        help_text="user's address")

    # When the profile was created and/or updated
    created_at = models.DateTimeField(
        auto_now_add=True,
        help_text="Time at which the profile was created")
    updated_at = models.DateTimeField(
        auto_now=True,
        help_text="Time at which the profile was updated")

    def __str__(self):
        """
        Function for converting the model into a string. This will allow us
        to have the name show up in the django admin UI
        """
        return self.user.__str__()


class RSVP(models.Model):
    """
    RSVP model. One RSVP is noted by the invite address, and will have 1+
    RSVPPerson
    """

    # Link back to the user. We can also use this to determine if the RSVP
    #   is claimed, since if it is None then we know it's not claimed
    profile = models.OneToOneField(
        Profile,
        null=True,
        on_delete=models.SET_NULL,
        related_name='rsvp',
        help_text='Profile with which the RSVP is associated')

    # Name for whom the RSVP is intended. This is more for internal record-
    #   keeping purposes than anything else
    last_names = models.CharField(
        max_length=256,
        help_text="Comma-separated last names of the people the RSVP is intended for")

    # Address where the invitation was sent. This is used to authenticate the
    #   user's attmept to claim the RSVP
    invite_address = models.CharField(
        max_length=256,
        help_text="Address invitation was sent to")

    # When the RSVP was created and/or updated
    created_at = models.DateTimeField(
        auto_now_add=True,
        help_text="Time at which the profile was created")
    updated_at = models.DateTimeField(
        auto_now=True,
        help_text="Time at which the profile was updated")

    # RSVP Comment
    comment = models.TextField(
        blank=True,
        help_text="RSVP Comment")

    def __str__(self):
        """
        Function for converting the model into a string. This will allow us
        to have the name show up in the django admin UI
        """
        return self.last_names

class RSVPPerson(models.Model):
    """
    RSVP Person. The actual RSVP Data
    """

    # Link back to the RSVP
    rsvp = models.ForeignKey(
        'RSVP',
        related_name='rsvp_person',
        on_delete=models.CASCADE,
        help_text='Link back to overall RSVP')

    # Person's Name
    name = models.CharField(
        max_length=128,
        help_text="RSVP Person's name")

    # Person's event attendance status
    is_attending_rehearsal = models.NullBooleanField(
        default=None,
        help_text="RSVP Status for Rehearsal Dinner")
    is_attending_wedding = models.NullBooleanField(
        default=None,
        help_text="RSVP Status for Sunday Wedding Ceremony & Reception")

    # Person's Child status.
    is_child = models.NullBooleanField(
        default=None,
        help_text="TRUE if person is a child")

    # Person's dietary restrictions
    dietary_kosher = models.BooleanField(
        default=False,
        help_text='Kosher Dietary Restriction')
    dietary_vegetarian = models.BooleanField(
        default=False,
        help_text='Vegetarian Dietary Restriction')
    dietary_vegan = models.BooleanField(
        default=False,
        help_text='Vegan Dietary Restriction')
    dietary_gluten_free = models.BooleanField(
        default=False,
        help_text='Gluten-Free Dietary Restriction')
    dietary_other = models.CharField(
        blank=True,
        max_length=256,
        help_text='Other Dietary Restrictions')

    # Person's special requests field
    special_requests = models.CharField(
        blank=True,
        max_length=256,
        help_text='Any other special requests for the person')

    # Table placement for the person
    table = models.IntegerField(
        default=0,
        help_text="Seating assignment for the guest")

    # When the profile was created and/or updated
    created_at = models.DateTimeField(
        auto_now_add=True,
        help_text="Time at which the profile was created")
    updated_at = models.DateTimeField(
        auto_now=True,
        help_text="Time at which the profile was updated")

    def __str__(self):
        """
        Function for converting the model into a string. This will allow us
        to have the name show up in the django admin UI
        """
        return self.name



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.contrib import admin

# Import our models
from .models import (
    Profile, RSVP, RSVPPerson
)

class ProfileAdmin(admin.ModelAdmin):
    """
    Admin for the profile
    """
    list_display = ('user', 'address', 'rsvp')

class RSVPPersonInline(admin.TabularInline):
    """
    Class to inline RSVP Persons within the RSVP in the admin UI
    """
    model = RSVPPerson

class RSVPAdmin(admin.ModelAdmin):
    """
    RSVP Admin class
    """

    # Fields to show in table view
    list_display = ('last_names', 'invite_address', 'profile')

    # Inlines to add
    inlines = [
        RSVPPersonInline,
    ]

    def get_form(self, request, obj=None, **kwargs):
        form = super(RSVPAdmin, self).get_form(request, obj, **kwargs)
        form.base_fields['profile'].required = False
        return form

class RSVPPersonAdmin(admin.ModelAdmin):
    """
    RSVP Person Admin class
    """
    list_display = ('name', 'rsvp', 'is_attending_rehearsal', 'is_attending_wedding')

# Hook them up to the admin UI
admin.site.register(Profile, ProfileAdmin)
admin.site.register(RSVP, RSVPAdmin)
admin.site.register(RSVPPerson, RSVPPersonAdmin)



from django.conf.urls import url, include
from django.http import Http404
from wedsite.views import (
    StaticView, StaticViewNoAuth, RSVPView, CreateAccountView
)
from django.contrib.auth.views import (
    LoginView, PasswordResetView, PasswordChangeView, PasswordChangeDoneView,
    LogoutView)
from wedsite.conf import settings

from . import views

def get_url(uri, name):
    """
    Given a URI and a name will create the url object for the
    urlpatterns list
    """
    template = "{}.html".format(name)

    if settings.WEDSITE_JSON["access"][name] == "all":
        view = StaticViewNoAuth.as_view(template=template)
    elif settings.WEDSITE_JSON["access"][name] == "login":
        view = StaticView.as_view(template=template)
    else:
        # TODO actual styled 404 page
        view = StaticViewNoAuth.as_view(template="404.html")

    return url(r'^{}$'.format(uri), view, name=name)

urlpatterns = [

    # Top-level URL serves the main index view
    get_url('', 'index'),
    get_url('contact', 'contact'),
    get_url('story', 'story'),
    get_url('wedding', 'wedding'),
    get_url('events', 'events'),
    get_url('travel', 'travel'),
    get_url('explore', 'explore'),
    get_url('gifts', 'gifts'),
    get_url('team', 'team'),
    get_url('traditions', 'traditions'),
    url(r'^rsvp$', RSVPView.as_view(), name='rsvp'),
    url(r'^create-account$', CreateAccountView.as_view(), name='create-account'),
]



from django.core.management.base import BaseCommand, CommandError
from django.contrib.auth.models import User
from django.shortcuts import render
from django.core.mail import EmailMultiAlternatives
from django.template.loader import get_template
from django.urls import set_script_prefix
from wedsite.conf import settings

class Command(BaseCommand):
    help = 'Sends a summary email to all users'

    def get_rsvp_persons(self, user):
        """
        Gets the RSVP for a user
        """
        try:
            rsvp = user.profile.rsvp
        except:
            print ("Error getting RSVP for {} {}".format(user.first_name, user.last_name))
            return []

        return rsvp.rsvp_person.all()

    def get_attending_rehearsal(self, user):
        result = []

        people = self.get_rsvp_persons(user)
        for person in people:
            if person.is_attending_rehearsal:
                result.append(person)

        return result

    def get_attending_wedding(self, user):
        result = []

        people = self.get_rsvp_persons(user)
        for person in people:
            if person.is_attending_wedding:
                result.append(person)

        return result

    def handle(self, *args, **options):
        '''send email via mailgun'''

        # Need a fake request to help us set up all the links in the email to work
        #fake_request = FakeHttpRequest()

        subject = "{} and {}'s Wedding: Update".format(settings.WEDSITE_JSON['gride']['first_name'], settings.WEDSITE_JSON['broom']['first_name'])
        from_email = "{} and {}'s Wedding<wedding@mg.wedsite.io>".format(settings.WEDSITE_JSON['gride']['first_name', settings.WEDSITE_JSON['broom']['first_name']])

        for user in User.objects.all():

            # Figure out if they're attending rehearsal/wedding
            attending_rehearsal = self.get_attending_rehearsal(user)
            attending_wedding = self.get_attending_wedding(user)
            text_template = get_template("email/email.txt")
            html_template = get_template("email/email.html")

            # Only send the message if they're attending the wedding
            if attending_wedding:

                template_info = {
                    "first_name": user.first_name,
                    "last_name": user.last_name,
                    "attending_rehearsal": self.get_attending_rehearsal(user),
                    "attending_wedding": self.get_attending_wedding(user),
                }

                text_content = text_template.render(template_info)
                html_content = html_template.render(template_info)
                print ("{} {}".format(user.first_name, user.last_name))
                msg = EmailMultiAlternatives(subject, text_content, from_email, [user.email])
                msg.attach_alternative(html_content, "text/html")
                msg.send()